/*
 *  File: ViewstateSaver.java 
 *  Copyright (c) 2004-2007  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.jaret.util.ui.timebars.addon;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.ui.IMemento;

import de.jaret.util.date.JaretDate;
import de.jaret.util.ui.timebars.model.HierarchicalTimeBarModel;
import de.jaret.util.ui.timebars.model.TimeBarNode;
import de.jaret.util.ui.timebars.swt.TimeBarViewer;

/**
 * Implementation of the viewstate saver for a timebarviewer. Saves and restores
 * <ul>
 * <li>Expanded state for hierarchical models</li>
 * <li>vertical scroll position</li>
 * <li>start date</li>
 * </ul>
 * 
 * @author kliem
 * @version $Id: ViewstateSaver.java 664 2007-12-10 21:40:08Z kliem $
 */
public class ViewstateSaver implements IViewstateSaver {
    /** key for storing the first row. */
    private static final String FIRSTROW = "firstrow";
    /** key for storing the first row offset. */
    private static final String FIRSTROWOFFSET = "firstrowoffset";
    /** key for storing the start date. */
    private static final String STARTDATE = "startdate";
    /** can be used as a singleton. */
    private static ViewstateSaver _instance = new ViewstateSaver();

    /**
     * The ViewStateSaver can be used as a singleton.
     * 
     * @return instance
     */
    public static ViewstateSaver getInstance() {
        return _instance;
    }

    /**
     * {@inheritDoc}
     */
    public void restoreViewstate(TimeBarViewer timeBarViewer, IHierarchyIdService idService, String id, IMemento memento) {
        if (memento == null) {
            return;
        }
        if (id != null) {
            memento = memento.getChild(id);
        }
        if (memento == null) {
            return;
        }
        
        // vertical scroll position
        Integer firstRow = memento.getInteger(FIRSTROW);
        if (firstRow != null) {
            timeBarViewer.setFirstRowDisplayed(firstRow);
        }
        Integer firstRowOffset = memento.getInteger(FIRSTROWOFFSET);
        if (firstRowOffset != null) {
            timeBarViewer.setFirstRowOffset(firstRowOffset);
        }
        
        // horizontal = startDate
        String startDateString = memento.getString(STARTDATE);
        if (startDateString != null) {
            JaretDate d = new JaretDate(startDateString);
            timeBarViewer.setStartDate(d);
        }
        
        // for hierarchical models try to restore expanded states
        if (timeBarViewer.getHierarchicalModel() != null) {
            int nr = 0;
            String nodeId = "x";
            while (nodeId != null) {
                nodeId = memento.getString("node" + nr++);
                if (nodeId != null) {
                    boolean state = memento.getInteger(nodeId) == 1;
                    TimeBarNode node = idService.getNode(timeBarViewer.getHierarchicalModel(), nodeId);
                    if (node != null) {
                        timeBarViewer.getHierarchicalViewState().setExpanded(node, state);
                    }
                }
            }
        }

    }

    /**
     * {@inheritDoc}
     */
    public void saveViewstate(TimeBarViewer timeBarViewer, IHierarchyIdService idService, String id, IMemento memento) {
        // create child for the given id if the id is not null
        if (id != null) {
            memento = memento.createChild(id);
        }

        // vertical scroll position
        memento.putInteger(FIRSTROW, timeBarViewer.getFirstRowDisplayed());
        memento.putInteger(FIRSTROWOFFSET, timeBarViewer.getFirstRowOffset());
        // horizontal = startDate
        memento.putString(STARTDATE, timeBarViewer.getStartDate().toString());
        
        // save expanded state for every node
        if (timeBarViewer.getHierarchicalModel() != null) {
            HierarchicalTimeBarModel model = timeBarViewer.getHierarchicalModel();
            List<TimeBarNode> allNodes = getAllTimeBarNodes(timeBarViewer);
            int nr = 0;
            for (TimeBarNode node : allNodes) {
                String nodeId = idService.getId(model, node);
                memento.putString("node" + nr++, nodeId);
//                System.out.println("NODE ID "+nodeId);
                memento.putInteger(nodeId, timeBarViewer.getHierarchicalViewState().isExpanded(node) ? 1 : 0);
            }
        }
    }

    /**
     * Retrieve all nodes in a flat list.
     * 
     * @param tbv timebarviewer
     * @return flat list of nodes
     */
    private List<TimeBarNode> getAllTimeBarNodes(TimeBarViewer tbv) {
        List<TimeBarNode> result = new ArrayList<TimeBarNode>();
        TimeBarNode root = tbv.getHierarchicalModel().getRootNode();
        result.add(root);
        collectNodes(root, result);
        return result;
    }

    /**
     * Collect nodes recursively.
     * 
     * @param node node that will be added to the result and of which the children will be processed
     * @param list resulting list
     */
    private void collectNodes(TimeBarNode node, List<TimeBarNode> list) {
        for (TimeBarNode n : node.getChildren()) {
            list.add(n);
            collectNodes(n, list);
        }
    }

}
