/*
 *  File: TableTimeBarModel.java 
 *  Copyright (c) 2004-2007  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.jaret.util.ui.timebars.addon.tablesynchronizer;

import java.util.HashMap;
import java.util.Map;

import de.jaret.util.date.Interval;
import de.jaret.util.ui.table.model.DefaultJaretTableModel;
import de.jaret.util.ui.timebars.model.TimeBarModel;
import de.jaret.util.ui.timebars.model.TimeBarModelListener;
import de.jaret.util.ui.timebars.model.TimeBarRow;

/**
 * Implementation of a JaretTableModel wrapping a TimeBarModel.
 * 
 * @author kliem
 * @version $Id: TableTimeBarModel.java 536 2007-08-19 10:49:10Z olk $
 */
public class TableTimeBarModel extends DefaultJaretTableModel implements TimeBarModelListener {
    /** underlying timebar model. */
    protected TimeBarModel _timeBarModel;
    /** counter for id generation. */
    protected int _idCounter = 0;
    /** map mapping ids to timebar rows. */
    protected Map<String, TimeBarRow> _idTimeBarRowMap = new HashMap<String, TimeBarRow>();
    /** map supporting the lookup of tabletimebar rows for timebar rows. */
    protected Map<TimeBarRow, TableTimeBarRow> _tbrTottbrMap = new HashMap<TimeBarRow, TableTimeBarRow>();

    /**
     * Construct a TableTimeBarModel for a TimeBarModel.
     * 
     * @param timeBarModel the model to be wrapped.
     */
    public TableTimeBarModel(TimeBarModel timeBarModel) {
        _timeBarModel = timeBarModel;

        _timeBarModel.addTimeBarModelListener(this);

        // init existing rows
        for (int i = 0; i < timeBarModel.getRowCount(); i++) {
            TimeBarRow tbRow = timeBarModel.getRow(i);
            TableTimeBarRow ttbr = new TableTimeBarRow(getId(tbRow), tbRow);
            addRow(ttbr);
            _tbrTottbrMap.put(tbRow, ttbr);
        }
    }

    /**
     * Lookup a TableTimeBarRow for a timebar row.
     * 
     * @param tbRow timebar row
     * @return the wrapping TableTiemBarRow
     */
    public TableTimeBarRow getTableTimeBarRow(TimeBarRow tbRow) {
        return _tbrTottbrMap.get(tbRow);
    }

    /**
     * Generate an id for a timebar row.
     * 
     * @param timeBarRow timebar row
     * @return generated id
     */
    protected String getId(TimeBarRow timeBarRow) {
        String id = "id" + (_idCounter++);
        _idTimeBarRowMap.put(id, timeBarRow);
        return id;
    }

    /**
     * {@inheritDoc}
     */
    public void elementAdded(TimeBarModel model, TimeBarRow row, Interval element) {
        fireRowChanged(getTableTimeBarRow(row));
    }

    /**
     * {@inheritDoc}
     */
    public void elementChanged(TimeBarModel model, TimeBarRow row, Interval element) {
        fireRowChanged(getTableTimeBarRow(row));
    }

    /**
     * {@inheritDoc}
     */
    public void elementRemoved(TimeBarModel model, TimeBarRow row, Interval element) {
        fireRowChanged(getTableTimeBarRow(row));
    }

    /**
     * {@inheritDoc}
     */
    public void headerChanged(TimeBarModel model, TimeBarRow row, Object newHeader) {
        fireRowChanged(getTableTimeBarRow(row));
    }

    /**
     * {@inheritDoc}
     */
    public void modelDataChanged(TimeBarModel model) {
        fireTableDataChanged();
    }

    /**
     * {@inheritDoc}
     */
    public void rowAdded(TimeBarModel model, TimeBarRow tbRow) {
        TableTimeBarRow ttbr = new TableTimeBarRow(getId(tbRow), tbRow);
        int index = getIndexForTBR(model, tbRow);
        addRow(index, ttbr);
        _tbrTottbrMap.put(tbRow, ttbr);
    }

    /**
     * Helper method to find the index of a timebar row in the model.
     * 
     * @param model timebar model
     * @param tbRow row to find
     * @return index or -1
     */
    private int getIndexForTBR(TimeBarModel model, TimeBarRow tbRow) {
        for (int i = 0; i < model.getRowCount(); i++) {
            if (model.getRow(i).equals(tbRow)) {
                return i;
            }
        }
        return -1;
    }

    /**
     * {@inheritDoc}
     */
    public void rowDataChanged(TimeBarModel model, TimeBarRow row) {
        fireRowChanged(getTableTimeBarRow(row));
    }

    /**
     * {@inheritDoc}
     */
    public void rowRemoved(TimeBarModel model, TimeBarRow tbRow) {
        TableTimeBarRow ttbr = getTableTimeBarRow(tbRow);
        remRow(ttbr);
    }

}
