/*
 *  File: MonthTableModel.java 
 *  Copyright (c) 2004-2007  Peter Kliem (Peter.Kliem@jaret.de)
 *  A commercial license is available, see http://www.jaret.de.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.jaret.examples.timebars.calendar.model;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import de.jaret.util.date.Interval;
import de.jaret.util.date.JaretDate;
import de.jaret.util.ui.table.model.AbstractJaretTableModel;
import de.jaret.util.ui.table.model.IColumn;
import de.jaret.util.ui.table.model.IRow;
import de.jaret.util.ui.timebars.model.TimeBarModel;
import de.jaret.util.ui.timebars.model.TimeBarModelListener;
import de.jaret.util.ui.timebars.model.TimeBarRow;

/**
 * A table model for the jaret table projecting a month from a calendar model (timebarmodel) to a table with weekday
 * columns.
 * 
 * @author kliem
 * @version $Id: MonthTableModel.java 716 2008-02-18 23:21:09Z kliem $
 */
public class MonthTableModel extends AbstractJaretTableModel {
    /** underlying calendar model. */
    protected CalendarModel _calendarModel;
    /** current month (0 based!). */
    protected int _month;
    /** current year. */
    protected int _year;

    /** calendar used for calculations. */
    protected Calendar _calendar = GregorianCalendar.getInstance();

    /** the columns corresponding to the weekdays. */
    protected List<IColumn> _columns = new ArrayList<IColumn>();

    /** rows corresponding to weeks. */
    protected List<IRow> _rows = new ArrayList<IRow>();

    /**
     * Create the model for a given month.
     * 
     * @param calendarModel calendar model to project
     * @param month month (0 based)
     * @param year year
     */
    public MonthTableModel(CalendarModel calendarModel, int month, int year) {
        _calendarModel = calendarModel;
        setMonth(month, year);

        createColumns();

        // add a listener to the calendar model for watching changes
        // TODO this is quick coded and always doe a full refresh
        calendarModel.addTimeBarModelListener(new TimeBarModelListener(){

            public void elementAdded(TimeBarModel model, TimeBarRow row, Interval element) {
                fireTableDataChanged();
            }

            public void elementChanged(TimeBarModel model, TimeBarRow row, Interval element) {
                fireTableDataChanged();
                
            }

            public void elementRemoved(TimeBarModel model, TimeBarRow row, Interval element) {
                fireTableDataChanged();
                
            }

            public void headerChanged(TimeBarModel model, TimeBarRow row, Object newHeader) {
                fireTableDataChanged();
            }

            public void modelDataChanged(TimeBarModel model) {
                fireTableDataChanged();
            }

            public void rowAdded(TimeBarModel model, TimeBarRow row) {
                fireTableDataChanged();
            }

            public void rowDataChanged(TimeBarModel model, TimeBarRow row) {
                fireTableDataChanged();
            }

            public void rowRemoved(TimeBarModel model, TimeBarRow row) {
                fireTableDataChanged();
            }
            
        });
        
    }

    /**
     * Creates the weekday columns.
     */
    private void createColumns() {
        IColumn[] weekDayColumns = new IColumn[7];
        for (int i = 1; i < 8; i++) {
            WeekDayColumn col = new WeekDayColumn(this, i);
            weekDayColumns[getWeekdayPos(i)] = col;
        }
        for (int i = 0; i < 7; i++) {
            _columns.add(weekDayColumns[i]);
        }
    }

    /**
     * Set the projected month.
     * 
     * @param month month (0 based)
     * @param year year
     */
    public void setMonth(int month, int year) {
        if (_month != month || _year != year) {
            _month = month;
            _year = year;

            _calendar.set(Calendar.YEAR, year);
            _calendar.set(Calendar.MONTH, month);
            _calendar.set(Calendar.DAY_OF_MONTH, 1);

            int week = _calendar.get(Calendar.WEEK_OF_YEAR);

            _rows.clear();

            for (int i = 0; i < 5; i++) {
                _rows.add(new CalendarWeekRow(this, year, week + i));
            }
        }
    }

    /**
     * Retrieve the month.
     * 
     * @return month (0 based)
     */
    public int getMonth() {
        return _month;
    }

    /**
     * Retrieve the current year.
     * 
     * @return year of the currently mapped month
     */
    public int getYear() {
        return _year;
    }

    /**
     * {@inheritDoc} Adding columns is not supported.
     */
    public void addColumn(IColumn column) {
    }

    /**
     * {@inheritDoc}
     */
    public IColumn getColumn(int idx) {
        return _columns.get(idx);
    }

    /**
     * {@inheritDoc}
     */
    public int getColumnCount() {
        return _columns.size();
    }

    /**
     * {@inheritDoc}
     */
    public IRow getRow(int idx) {
        return _rows.get(idx);
    }

    /**
     * {@inheritDoc}
     */
    public int getRowCount() {
        return _rows.size();
    }

    /**
     * Retrieve the index of the column for a weekday.
     * 
     * @param weekday coded weekday (1-7)
     * @return column position in the day grid (0-6)
     */
    private int getWeekdayPos(int weekday) {
        int firstDay = _calendar.getFirstDayOfWeek();
        int pos = weekday - firstDay;
        if (pos < 0) {
            pos += 7;
        }
        return pos;
    }

    /**
     * Retrive the underlying calendar model.
     * 
     * @return the calendar model
     */
    public CalendarModel getCalendarModel() {
        return _calendarModel;
    }

    /**
     * Retrieve the column that may correspond to the date.
     * 
     * @param date date
     * @return column with mathcing weekday
     */
    public IColumn getColumnForDate(JaretDate date) {
        return _columns.get(getWeekdayPos(date.getDayOfWeek()));
    }

    /**
     * Retrieve the row that matches the date.
     * 
     * @param date date
     * @return row or <code>null</code> if no row could be found for the date
     */
    public IRow getRowForDate(JaretDate date) {
        for (IRow row : _rows) {
            CalendarWeekRow cwr = (CalendarWeekRow) row;
            if (cwr.getWeek() == date.getWeekOfYear() && cwr.getYear() == date.getYear()) {
                return row;
            }
        }
        return null;
    }

}
